--- cgi-bin/openwebmail/shares/mailfilter.pl	Sun Apr 11 06:38:33 2004
+++ cgi-bin/openwebmail/shares/mailfilter.pl	Wed Apr 14 03:08:33 2004
@@ -1,6 +1,9 @@
 #
 # mailfilter.pl - mail filter routines
 #
+# 2004/04/09 ateslik.AT.users.sourceforge.net
+#            integrate SpamAssassin filtering
+#
 # 2001/11/13 Ebola.AT.turtle.ee.ncku.edu.tw
 #            tung.AT.turtle.ee.ncku.edu.tw
 #
@@ -130,6 +133,25 @@
       return -6;
    }
 
+   my $spamtest = '';
+   if ($config{'enable_spamassassin'} && $prefs{'sa_enable'}) {
+      eval "use Mail::SpamAssassin;";
+      if ($@) {
+         $lang_err{'couldnt_load'} =~ s/\@\@\@MODULENAME\@\@\@/'Mail::SpamAssassin'/;
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_load'} !");
+      }
+
+      my %spamtestconfig = (   userprefs_filename => dotpath('spamassassin_user_prefs'),
+                                    userstate_dir => dotpath('webmail'),
+                                  dont_copy_prefs => 1,
+                             home_dir_for_helpers => dotpath('webmail') );
+
+      $spamtestconfig{'local_tests_only'} = 1 if ($prefs{'sa_local_tests_only'});
+
+      $spamtest = Mail::SpamAssassin->new(\%spamtestconfig);
+      $spamtest->compile_now(1);
+   }
+
    foreach my $messageid (@allmessageids) {
       next if ($messageid=~/^DUP\d+\-/);        # skip duplicated msg in src folder
       my @attr = get_data_attr($FDB{$messageid});
@@ -455,7 +477,35 @@
             }
          } # end of if enable_smartfilter
 
-         # 3. mark to be moved message as zap
+         # 3. Check what SpamAssassin thinks of this message
+         if ($config{'enable_spamassassin'} && $prefs{'sa_enable'} && $to_be_moved != 1) {
+            if ($currmessage eq "") {
+               seek($folderhandle, $attr[$_OFFSET], 0);
+               read($folderhandle, $currmessage, $attr[$_SIZE]);
+            }
+            my $status = $spamtest->check_message_text($currmessage);
+            if ($status->is_spam()) {
+               if ($prefs{'sa_move_spam'}) {
+                  my $appended = undef;
+                  if ($prefs{'sa_spam_destination'} eq $folder) {
+                     $reserved_in_folder=1;
+                  } else {
+                     $appended=append_message_to_folder($messageid, $folderhandle,
+         				\@attr, \$currmessage, $user, $prefs{'sa_spam_destination'});
+                  }
+                  unless ($reserved_in_folder) {
+                     if ($appended>=0) {
+                        $to_be_moved=1;
+                     } else {
+                        $ioerr++;
+                     }
+                  }
+               }
+            }
+            $status->finish(); # cleanup $status object
+         }
+
+         # 4. mark to be moved message as zap
          if ($to_be_moved) {
             $attr[$_STATUS].='Z' if ($attr[$_STATUS]!~/Z/i);
             index_msg_attr(\%FDB, $messageid, @attr);
@@ -662,4 +712,114 @@
 }
 ########## END FILTERMESSAGE2 ####################################
 
+########## SABAYESLEARN ###########################################
+sub sabayeslearn {
+   unless ($config{'enable_spamassassin'} && $config{'sa_enable_bayesian'} && $prefs{'sa_enable'} && $prefs{'sa_use_bayes'}) {
+      return;
+   };
+
+   my ($r_messageids,$folder)=@_;
+
+   # will we go over our quota if we move messages around?
+   if ($quotalimit>0 && $quotausage>$quotalimit && $prefs{'sa_spam_destination'} ne "DELETE" && $prefs{'sa_move_spam'}) {
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'quotahit_alert'}");
+   }
+
+   # get our folder info
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $folder);
+   if (! -f "$folderfile" ) {
+      openwebmailerror(__FILE__, __LINE__, "$folderfile $lang_err{'doesnt_exist'}");
+   }
+
+   # get our destination folder info
+   my ($dstfile, $dstdb)=get_folderpath_folderdb($user, $prefs{'sa_spam_destination'});
+   if ($prefs{'sa_spam_destination'} ne 'DELETE' && ! -f "$dstfile" ) {
+      open (F,">>$dstfile") or
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $lang_err{'destination_folder'} $dstfile! ($!)");
+      close(F);
+   }
+
+   # lock our folders for reading and writing
+   ow::filelock::lock("$folderfile", LOCK_EX|LOCK_NB) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
+   if ($prefs{'sa_spam_destination'} ne 'DELETE') {
+      ow::filelock::lock($dstfile, LOCK_EX|LOCK_NB) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $dstfile!");
+   }
+
+   # initialize learner
+   eval "use Mail::SpamAssassin;";
+   if ($@) {
+      $lang_err{'couldnt_load'} =~ s/\@\@\@MODULENAME\@\@\@/'Mail::SpamAssassin'/;
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_load'} !");
+   }
+   my $bayeslearn = Mail::SpamAssassin->new( {   userprefs_filename => dotpath('spamassassin_user_prefs'),
+                                                      userstate_dir => dotpath('webmail'),
+                                                    dont_copy_prefs => 1,
+                                               home_dir_for_helpers => dotpath('webmail') } );
+   $bayeslearn->compile_now(1);
+
+   my $learntype = param('action') eq 'learnspam' ? 1 : undef;
+
+   my $folderhandle=do { local *FH };
+   open ($folderhandle, "+<$folderfile") or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderfile!");
+   foreach my $messageid (@{$r_messageids}) {
+      my @attr=get_message_attributes($messageid, $folderdb);
+      my $currmessage = '';
+      if ($currmessage eq "") {
+         seek($folderhandle, $attr[$_OFFSET], 0);
+         read($folderhandle, $currmessage, $attr[$_SIZE]);
+      }
+
+      # encapsulate message in a Mail::Audit object
+      my @lines = split (/^/m, $currmessage);
+      my $mail_obj = Mail::SpamAssassin::NoMailAudit->new ('data' => \@lines);
+
+      # learn this mail
+      $bayeslearn->init_learner();
+      $bayeslearn->learn($mail_obj, undef, $learntype, undef);
+      $bayeslearn->finish_learner();
+   }
+   $bayeslearn->rebuild_learner_caches();
+   close($folderhandle);
+
+   # move the spam if requested
+   my $op = $prefs{'sa_spam_destination'} eq 'DELETE' ? 'delete' : 'move';
+   my $counted=0;
+   if ($prefs{'sa_move_spam'} && $learntype) {
+      if ($op eq "delete") {
+         $counted=operate_message_with_ids($op, $r_messageids, $folderfile, $folderdb);
+      } else {
+         $counted=operate_message_with_ids($op, $r_messageids, $folderfile, $folderdb, $dstfile, $dstdb);
+      }
+      $counted=folder_zapmessages($folderfile, $folderdb);
+   }
+
+   # unlock the folders
+   ow::filelock::lock($dstfile, LOCK_UN);
+   ow::filelock::lock($folderfile, LOCK_UN);
+
+   # write the logs
+   if ($counted>0){
+      my $msg;
+      if ( $op eq 'move') {
+         $msg="move message - move $counted learned spam msgs from $folder to $prefs{'sa_spam_destination'} - ids=".join(", ", @{$r_messageids});
+      } else {
+         $msg="delete message - delete $counted learned spam msgs from $folder - ids=".join(", ", @{$r_messageids});
+        # recalc used quota for del if user quotahit
+        if ($quotalimit>0 && $quotausage>$quotalimit) {
+           $quotausage=(ow::quota::get_usage_limit(\%config, $user, $homedir, 1))[2];
+        }
+      }
+      writelog($msg);
+      writehistory($msg);
+   } elsif ($counted==-1) {
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'inv_msg_op'}");
+   } elsif ($counted==-2) {
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderfile");
+   } elsif ($counted==-3) {
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $dstfile!");
+   }
+
+   return;
+}
+########## END SABAYESLEARN #######################################
 1;
